% KDEDEMO3    Demonstrate UIControls with 3D Kernel Density Estimates.
%
%             Example: data=randn(500,3); (optional - defines data)
%                      kdedemo3
%
%             Menus: (a) DATA - A choice of three 2D data sets.
%                    (b) COLOUR - A choice of several colormaps.
%
%             Pop-ups: (a) PLOT - Choice of Surface, Contour or Percent Contour plots.
%                      (c) HSELECT - A choice of several data-based h selection strategies.
%
%             Check-boxes: SHADNG - Turns interpolated shading on/off.
%
%             Push buttons: (a) END - Quit demonstration,
%                           (b) HELP - Output these comments.
%
%             Sliders: (a) LR - Sets azimuth angle of view,
%                      (b) UD - Sets elevation angle of view.
%
%             Editable text: (a) H - Click here to enter a value for H.  Values 
%                                of H=0 cause the routine to use an 'optimal' 
%                                value.  If entering >1 value use vector notation, 
%                                eg [2,1] or [2,0].
%                            (b) CONTOUR PERCENTAGES - eg [25,50,75] or [50,100] or
%                                [10:10:90]. 
% 
%             Christian C. Beardah 1995  

echo off

load bvdata;

if exist('data')==0,
  data=wvglass(:,1:3);
  import=0;
  else,
    data=data(:,1:3);
    import=1;
end;

if exist('nmesh')==0,
  nmesh=32;
end;

xf=0.5; yf=0.5; zf=0.5;

mindata=min(data);
maxdata=max(data);

xmin=mindata(1);
ymin=mindata(2);
zmin=mindata(3);

xmax=maxdata(1);
ymax=maxdata(2);
zmax=maxdata(3);

xp=xmin+xf*(xmax-xmin);
yp=xmin+yf*(ymax-ymin);
zp=xmin+zf*(zmax-zmin);

% Initialise

kernel='knorm';
azim=-37.5;
elev=30;
h=[hns(data(:,1),kernel) hns(data(:,2),kernel) hns(data(:,3),kernel)];
pc=75;
sstr='y';

figure('Name','Tri-variate KDE demo');

gcfpos = zeros(1,4);

[X,Y,Z,f]=kdplot3(data,kernel,nmesh,h,1);
d=kdest3d(data(:,1),data(:,2),data(:,3),data,kernel,h);

view(azim,elev);

% Callback strings

ends = 'clear global; clear; close;';

holds = ['if get(holdh,''value'')==1, hold on; else, hold off; set(holdh,''value'',0); end;'];

smooths = 'if get(smoothh,''value'')==1, shading interp; else, shading faceted; end; ';

dview2s = 'view(2); set(udc,''value'',90); set(lrc,''value'',0); azim=0; elev=90;';

dview3s = 'view(3); set(udc,''value'',30); set(lrc,''value'',-37.5); azim=-37.5; elev=30;';

sliders = ['xf=0.5; yf=0.5; zf=0.5; mindata=min(data); maxdata=max(data);',...
           'xmin=mindata(1); ymin=mindata(2); zmin=mindata(3);',...
           'xmax=maxdata(1); ymax=maxdata(2); zmax=maxdata(3);',...
           'set(xc,''value'',xf); set(yc,''value'',yf); set(zc,''value'',zf);'];

views = ['view(azim,elev); v=get(ploth,''value''); if v<3, xlabel(''x''); ylabel(''y''); zlabel(''z'');',...
         'title([''h = '',num2str(h(1)),'' , '',num2str(h(2)),'' , '',num2str(h(3))]); else,',...
         'xlabel(''Minimum distance from contour''); ylabel(''Density''); end;'];

plots = [holds,'set(zoomh,''value'',0); v=get(ploth,''value''); if v==1, xp=xmin+xf*(xmax-xmin); yp=ymin+yf*(ymax-ymin);',...
         'zp=zmin+zf*(zmax-zmin); slice(X,Y,Z,f,xp,yp,zp,nmesh);',...
          smooths,'elseif v==2, [outl,index,haxis]=pclvl3(pc,data,X,Y,Z,f,d,sstr); outl,',...
         'else, if exist(''outl''), plot(outl(:,6),outl(:,5),''wx''); elev=90; set(udc,''value'',elev);',...
         'for i=1:length(outl), text(outl(i,6),outl(i,5),int2str(outl(i,1))); end; else, set(ploth,''value'',1); end; end;',views];

zooms = ['v=get(zoomh,''value''); if v==1, if ~exist(''fz''), [Xz,Yz,Zz,fz]=kdplot3(data,kernel,nmesh,h,0,haxis); end;',...
         'pclvl3a(pc,data,Xz,Yz,Zz,fz,d,sstr);',views,'else,',plots,'end;'];

hselects = ['clear Xz Yz Zz fz; v=get(hselecth,''value''); if v==1, h=[hns(data(:,1),kernel) hns(data(:,2),kernel) hns(data(:,3),kernel)];',...
           'elseif v==2 & kernel==''knorm'', h=[hdpi(data(:,1),kernel,1) hdpi(data(:,2),kernel,1) hdpi(data(:,3),kernel,1)];',...
           'elseif v==3 & kernel==''knorm'', h=[hdpi(data(:,1),kernel,2) hdpi(data(:,2),kernel,2) hdpi(data(:,3),kernel,2)];',...
           'elseif v==4 & kernel==''knorm'', h=[hdpi(data(:,1),kernel,3) hdpi(data(:,2),kernel,3) hdpi(data(:,3),kernel,3)];',...
           'elseif v==5 & kernel==''knorm'', h=[hste(data(:,1),kernel) hste(data(:,2),kernel) hste(data(:,3),kernel)];',...
           'else, h=[hns(data(:,1),kernel) hns(data(:,2),kernel) hns(data(:,3),kernel)];',...  
           'set(hselecth,''value'',1); end; set(hvalh,''value'',h); [X,Y,Z,f,h]=kdplot3(data,kernel,nmesh,h,0); d=kdest3d(data(:,1),data(:,2),data(:,3),data,kernel,h);',plots];

hvals = ['h=str2num(get(hvalh,''string'')); if min(h)<0, disp(''ERROR - negative h'');',...
         'else, [X,Y,Z,f]=kdplot3(data,kernel,nmesh,h,0); d=kdest3d(data(:,1),data(:,2),data(:,3),data,kernel,h);',plots,'end;'];

pcvals = ['pc=str2num(get(pcvalh,''string'')); if min(pc)<0, disp(''ERROR - negative contour level'');',...
         'end;',zooms];

meshs = ['nmesh=str2num(get(meshh,''string'')); if nmesh<8, disp(''ERROR - mesh too small'');',...
         'else, [X,Y,Z,f]=kdplot3(data,kernel,nmesh,h,0);',plots,'end;'];

% Frames

frame1h = uicontrol('style','frame','units','normal','pos',[.84 .85 .15 .14]);

frame2h = uicontrol('style','frame','units','normal','pos',[.02 .02 .29 .06]);

% Check boxes

smoothh = uicontrol('style','checkbox','units','normal','pos',[.02 .87 .135 .05],...
  'string','Shading','call',smooths);
zoomh = uicontrol('style','checkbox','units','normal','pos',[.02 .2 .09 .05],...
  'string','Zoom','call',zooms);
holdh = uicontrol('style','checkbox','units','normal','pos',[.89 .72 .09 .05],...
  'string','Hold','call',holds);

% Pop-up menus

ploth = uicontrol('style','popupmenu','units','normal','pos',[.015 .94 .24 .08],...
  'string','Slice|Percent Contour|Outliers','call',plots);   
hselecth = uicontrol('style','popupmenu','units','normal','pos',[.8 .79 .19 .08],...
  'string','Normal Scale|DPI 1|DPI 2|DPI 3|STE2','call',hselects);   

% Menus

datamenuh = uimenu(gcf,'label','Data');

wvglassh = uimenu(datamenuh,'label','Win. Vessel (102)','callback',...
  ['data=wvglass(:,1:3);',sliders,hselects,...
   'set(wvglassh,''checked'',''on''),',...
   'set(sglassh,''checked'',''off''),',... 
   'set(lmglassh,''checked'',''off''),',... 
   'set(wprglassh,''checked'',''off''),',... 
   'set(cglassh,''checked'',''off'')']);
sglassh = uimenu(datamenuh,'label','Southampton (271)','callback',...
  ['data=sglass(:,1:3);',sliders,hselects,...
   'set(wvglassh,''checked'',''off''),',...
   'set(sglassh,''checked'',''on''),',... 
   'set(lmglassh,''checked'',''off''),',... 
   'set(wprglassh,''checked'',''off''),',... 
   'set(cglassh,''checked'',''off'')']);
lmglassh = uimenu(datamenuh,'label','Leic./Man. (105)','callback',...
  ['data=lmglass(:,1:3);',sliders,hselects,...
   'set(wvglassh,''checked'',''off''),',...
   'set(sglassh,''checked'',''off''),',... 
   'set(lmglassh,''checked'',''on''),',... 
   'set(wprglassh,''checked'',''off''),',... 
   'set(cglassh,''checked'',''off'')']);
wprglassh = uimenu(datamenuh,'label','Win. post-Roman (250)','callback',...
  ['data=wprglass(:,1:3);',sliders,hselects,...
   'set(wvglassh,''checked'',''off''),',...
   'set(sglassh,''checked'',''off''),',... 
   'set(lmglassh,''checked'',''off''),',... 
   'set(wprglassh,''checked'',''on''),',... 
   'set(cglassh,''checked'',''off'')']);
cglassh = uimenu(datamenuh,'label','Coppergate (234)','callback',...
  ['data=cglass(:,1:3);',sliders,hselects,...
   'set(wvglassh,''checked'',''off''),',...
   'set(sglassh,''checked'',''off''),',... 
   'set(lmglassh,''checked'',''off''),',... 
   'set(wprglassh,''checked'',''off''),',... 
   'set(cglassh,''checked'',''on'')']);

if import==0,
  set(wvglassh,'checked','on');
end;

sstrmenus = ['set(yellowh,''checked'',''off''),',...
             'set(magentah,''checked'',''off''),',...
             'set(cyanh,''checked'',''off''),',...
             'set(redh,''checked'',''off''),',...
             'set(greenh,''checked'',''off''),',...
             'set(whiteh,''checked'',''off''),',...
             'set(blackh,''checked'',''off''),',...
             'set(blueh,''checked'',''off'');'];

sstrmenuh = uimenu(gcf,'label','Cont. col.');

yellowh = uimenu(sstrmenuh,'label','Yellow','callback',...
  ['sstr=''y'';', sstrmenus, 'set(yellowh,''checked'',''on'')']);
magentah = uimenu(sstrmenuh,'label','Magenta','callback',...
  ['sstr=''m'';', sstrmenus, 'set(magentah,''checked'',''on'')']);
cyanh = uimenu(sstrmenuh,'label','Cyan','callback',...
  ['sstr=''c'';', sstrmenus, 'set(cyanh,''checked'',''on'')']);
redh = uimenu(sstrmenuh,'label','Red','callback',...
  ['sstr=''r'';', sstrmenus, 'set(redh,''checked'',''on'')']);
greenh = uimenu(sstrmenuh,'label','Green','callback',...
  ['sstr=''g'';', sstrmenus, 'set(greenh,''checked'',''on'')']);
whiteh = uimenu(sstrmenuh,'label','White','callback',...
  ['sstr=''w'';', sstrmenus, 'set(whiteh,''checked'',''on'')']);
blueh = uimenu(sstrmenuh,'label','Blue','callback',...
  ['sstr=''b'';', sstrmenus, 'set(blueh,''checked'',''on'')']);
blackh = uimenu(sstrmenuh,'label','Black','callback',...
  ['sstr=''k'';', sstrmenus, 'set(blackh,''checked'',''on'')']);

set(yellowh,'checked','on');

% Sliders

lrc = uicontrol('style','slider','units','normal','pos',[.385 .02 .25 .04],...
     'min',-90,'max',90,'value',azim,...
     'call',['azim = ceil(get(lrc,''value'')); set(lrc,''value'',azim);',views;]);
lrmin = uicontrol('style','text','units','normal','pos',[.34 .02 .04 .04],...
     'string',num2str(get(lrc,'min')));
lrmax = uicontrol('style','text','units','normal','pos',[.64 .02 .04 .04],...
     'string',num2str(get(lrc,'max')));

udc = uicontrol('style','slider','units','normal','pos',[.945 .36 .04 .25],...
     'min',0,'max',90,'value',elev,...
     'call',['elev = ceil(get(udc,''value'')); set(udc,''value'',elev);',views;]);
udmin = uicontrol('style','text','units','normal','pos',[.945 .32 .04 .035],...
     'string',num2str(get(udc,'min')));
udmax = uicontrol('style','text','units','normal','pos',[.945 .615 .04 .035],...
     'string',num2str(get(udc,'max')));

xc = uicontrol('style','slider','units','normal','pos',[.69 .1 .25 .04],...
     'min',0,'max',1,'value',xf,...
     'call',['xf = get(xc,''value''); set(xc,''value'',xf);',plots;]);
xmint = uicontrol('style','text','units','normal','pos',[.645 .1 .04 .04],...
     'string',num2str(get(xc,'min')));
xmaxt = uicontrol('style','text','units','normal','pos',[.945 .1 .04 .04],...
     'string',num2str(get(xc,'max')));

yc = uicontrol('style','slider','units','normal','pos',[.065 .1 .25 .04],...
     'min',-1,'max',0,'value',-yf,...
     'call',['yf = abs(get(yc,''value'')); set(yc,''value'',-yf);',plots;]);
ymint = uicontrol('style','text','units','normal','pos',[.02 .1 .04 .04],...
     'string','1');
ymaxt = uicontrol('style','text','units','normal','pos',[.32 .1 .04 .04],...
     'string','0');

zc = uicontrol('style','slider','units','normal','pos',[.02 .36 .04 .25],...
     'min',0,'max',1,'value',zf,...
     'call',['zf = get(zc,''value''); set(zc,''value'',zf);',plots;]);
zmint = uicontrol('style','text','units','normal','pos',[.02 .32 .04 .035],...
     'string',num2str(get(zc,'min')));
zmaxt = uicontrol('style','text','units','normal','pos',[.02 .615 .04 .035],...
     'string',num2str(get(zc,'max')));

% Push buttons

endh = uicontrol('style','push','units','normal','pos',[.91 .02 .08 .06],...
        'string','End','call',ends);
dview2h = uicontrol('style','push','units','normal','pos',[.71 .02 .08 .06],...
        'string','View2','call',dview2s);
dview3h = uicontrol('style','push','units','normal','pos',[.81 .02 .08 .06],...
        'string','View3','call',dview3s);

% Editable text

hvalh = uicontrol('style','edit','units','normal','pos',[.885 .92 .09 .04],...
        'string','0.00','min',0,'call',hvals);
hvalt = uicontrol('style','text','units','normal','pos',[.86 .92 .02 .04],...
        'string','h:');

set(hvalh,'string',['[',num2str(h(1)),',',num2str(h(2)),',',num2str(h(3)),']']);

pcvalh = uicontrol('style','edit','units','normal','pos',[.185 .025 .12 .04],...
        'string','75','min',0,'call',pcvals);
pcvalt = uicontrol('style','text','units','normal','pos',[.035 .025 .15 .04],...
        'string','Contour %:');

meshh = uicontrol('style','edit','units','normal','pos',[.92 .86 .05 .04],...
        'string',num2str(nmesh),'min',8,'call',meshs);
mesht = uicontrol('style','text','units','normal','pos',[.86 .86 .06 .04],...
        'string','mesh:');

colormenu;